var IB_SKIN = IB_SKIN || {};

/**
 * 音声
 * @param {any} path
 * @param {any} option
 */
IB_SKIN.Audio = function (path, option) {
    var thisAudio = this;
    this.path = path;
    this.totalTime = -1;
    this.currentTime = -1;
    this.option = option;
    this.isLoaded = false;

    // 事前にロード
    this.element = new Audio(path);

    this.element.addEventListener('loadedmetadata', function () {
        thisAudio.totalTime = thisAudio.element.duration;
        thisAudio.currentTime = 0;
        thisAudio.isLoaded = true;
    });
    this.element.addEventListener('timeupdate', function () {
        thisAudio.currentTime = thisAudio.element.currentTime;
    });
    this.element.loop = true;
    this.element.load();

    this.getDuration = function () {
        if (this.option.repeatCount) {
            return (this.totalTime * 1000) * this.option.repeatCount;
        }
        if (this.option.time) {
            return this.option.time * 1000;
        };
    }

    /**
     * 再生
     */
    this.play = function () {
        var sleep = function (period, decision, callback) {
            var interval = setInterval(function () {
                if (decision()) {
                    interval = clearInterval(interval);
                    callback();
                }
            }, period);
        }

        // ロード完了後に再生
        sleep(100, function () { return thisAudio.isLoaded; }, function () {
            thisAudio.element.play();
        });
    }

    /**
     * 一時停止
     */
    this.pause = function () {
        thisAudio.element.pause();
    }
}

/**
 * プレイヤー
 */
IB_SKIN.AudioPlayer = (function () {
    // 再生中音源
    this.playNow = null;
    // 再生キュー
    this.queue = [];
});

/**
 * 初期化処理
 * @param {any} audio
 */
IB_SKIN.AudioPlayer.prototype.initialize = function (audio) {
    var thisPlayer = this;
    this.playNow = audio;

    var endedProccess = function () {
        thisPlayer.playNow = null;
        var next = thisPlayer.queue.shift();
        if (next)
            thisPlayer.play(next);
    };
    audio.element.addEventListener('ended', endedProccess);
    audio.element.addEventListener('pause', endedProccess);
};

/**
 * 再生されているかどうか
 */
IB_SKIN.AudioPlayer.prototype.isPlay = function () {
    return this.playNow.currentTime > 0 && !this.playNow.paused &&
        !this.playNow.ended && this.playNow.readyState > 2;
};

/**
 * 再生します
 * @param {any} audio
 */
IB_SKIN.AudioPlayer.prototype.play = function (audio) {
    if (!audio || !audio.path) {
        console.error('MUST NEED IB_SKIN.Audio');
        return;
    }
    if (this.playNow != null) {
        // 再生中の場合、キューに積む
        this.queue.push(audio);
        return;
    }
    this._play(audio);
};

/**
 * 再生処理(private)
 * @param {any} audio
 */
IB_SKIN.AudioPlayer.prototype._play = function (audio) {
    var that = this;
    that.initialize(audio);

    if (!this.isPlay()) {
        audio.element.addEventListener('play', function () {
            setTimeout(function () {
                console.log("[SOUND] PLAY " + audio.path + " DURATION=", audio.getDuration());
                setTimeout(function () {
                    that.stop();
                }, audio.getDuration());
            }, 200);
        });
        audio.play();
    }
}

/**
 * 停止
 */
IB_SKIN.AudioPlayer.prototype.stop = function () {
    this.playNow.currentTime = this.playNow.duration;
    this.playNow.pause();
    // クリア
    this.playNow = null;
}