﻿(function (window) {
    /**
     * すべてのビュー
     */
    var allViews = 'ReplaceViewSettings';

    /**
     * 表示対象のオートビュー
     */
    var currentAutoView = (function () {
        let autoViewId = getParameterByName("autoViewId");
        let autoview = autoViewId
            ? allViews.filter(function (c) {
                return c.id == autoViewId;
            })[0]
            : allViews[0];
        console.info("[AUTOVIEW]ID=" + autoview.id + " NAME=" + autoview.name);
        return autoview;
    })();

    var IS_DISPLAY_MENU = 'IsDisplayMenu';

    /**
     * 表示対象のビューの一覧
     */
    var viewList = (function () {
        return currentAutoView.views;
    })();

    /**
     * 現在のビュー
     */
    var nowView = null;

    /**
     * クエリ文字列をもとに取得したビュー
     */
    var paramView = getQueryView();

    /**
     * ビューの読込回数
     */
    var loadViewCount = 0;

    /**
     * 最大ビュー読込回数
     * ビューを読込回数が規定回数に達した場合、リロードを行う。
     */
    var MAX_LOAD_VIEW_COUNT = 1000;

    // 初期処理
    window.addEventListener(
        "load",
        function () {
            //Chart描画用canvas/stage
            chartCanvas = document.getElementById("chartCanvas");

            // 通常、キャンバスが重なっている場合に、マウスイベントが最前面にしか当たらないので、
            // 最前面以外の要素にも順番にイベントを発火させる。
            var chartDiv = this.document.getElementById("chartDiv");
            chartDiv.addEventListener("mousemove", function (ev) {
                ev.preventDefault();

                var elements = chartDiv.children;
                for (let i = 0; i < elements.length; i++) {
                    triggerEvent(elements[i], ev);
                }
            });

            //描画用canvas/stage
            canvas = document.getElementById("mainCanvas");
            stage = new createjs.Stage("mainCanvas");
            stage.clear();

            //datatableインスタンス管理
            tables = [];

            window.removeEventListener("load", arguments.callee);
            var querystring = queryString = parseQueryStringToDictionary(
                location.href.split("?")[1] || ""
            )

            // メニューの表示設定によって、表示・非表示を切り替える。
            if (IS_DISPLAY_MENU || (!IS_DISPLAY_MENU && querystring['autoViewId'] == null && querystring['viewId'] == null)) {
                createMenu();
            } else {
                // 非表示
                const menu = document.querySelector(".menuDiv");
                menu.classList.add("no-display");
            }

            updateView();

            /** 検索ワード */
            document
                .getElementById("search-form")
                .addEventListener("input", function (e) {
                    const value = e.target.value;

                    // 全て非表示
                    for (const el of document.querySelectorAll(".autoview")) {
                        el.classList.add("no-display");
                    }

                    for (const el of document.querySelectorAll(".autoview a")) {
                        if (el.textContent.includes(value)) {
                            el.parentNode.classList.remove("no-display");
                        }
                    }
                });
        },
        false
    );

    // リサイズ時
    window.onresize = function () {
        adjustLayout();
    };

    /**
     * ビューの更新
     */
    function updateView() {
        // Clear Timer.
        IB_SKIN.clearAutoScrollTimer();

        if (window.CollectGarbage) {
            window.CollectGarbage();
        }

        nowView =
            (loadViewCount != 0 && paramView) || !paramView
                ? getNextView(nowView)
                : paramView;

        loadViewCount++;
        console.info(
            "[VIEW][" +
            ("    " + loadViewCount).slice(-4) +
            "] DISPLAY_VIEW VIEWID=" +
            nowView.viewId +
            " VIEW_NAME=" +
            nowView.name +
            " INTERVAL=" +
            nowView.interval +
            " IS_CYECLE_DISPLAY=" +
            nowView.isCycleDisplay +
            " PARAMETER=" +
            JSON.stringify(nowView.parameters)
        );
        loadView(nowView);

        // 一定回数読み込まれた時点で再読込を行う。
        if (loadViewCount >= MAX_LOAD_VIEW_COUNT) {
            console.log("REFRESH VIEW...");
            $.get(location.href).done(function (x) {
                location.reload(true);
            });
        }

        // Change title -> view name.
        document.title = nowView.name;

        if (nowView.interval > 0) {
            setTimeout(updateView, nowView.interval);
        }
    }

    /**
     * ビューを読み込みます。
     */
    function loadView(view) {
        // 不要なイベントハンドラが残らないよう削除する。
        resetCreateJSEventHandler();

        var script = document.getElementById("view_js");
        if (script) document.body.removeChild(script);

        script = document.createElement("script");
        script.src = createJsUrl(view);
        script.id = "view_js";
        script.onload = adjustLayout;
        document.body.appendChild(script);
    }

    function getQueryView() {
        if (!window.location.search.length) {
            return null;
        }

        // ビューIDの指定があれば、対象のビューのリストを生成
        var targetViews = [],
            viewId = getParameterByName("viewId"),
            queryString = parseQueryStringToDictionary(
                location.href.split("?")[1] || ""
            );

        for (var i = 0; i < allViews.length; i++) {
            for (var j = 0; j < allViews[i].views.length; j++) {
                var view = allViews[i].views[j];
                if (viewId != null && viewId == view.viewId) {
                    targetViews.push(view);
                }
            }
        }

        // Object To Array
        var params = [];
        for (var key in queryString) {
            if (key == "autoViewId" || key == "viewId") continue;

            params.push({
                name: decodeURIComponent(key),
                value: decodeURIComponent(queryString[key]),
            });
        }

        // パラメータが合致するものがあれば、そのビューを返す。
        for (var i = 0; i < targetViews.length; i++) {
            // パラメータの並び順がソート済み必要性有り
            if (JSON.stringify(params) == JSON.stringify(targetViews[i].parameters)) {
                return targetViews[i];
            }
        }

        return targetViews[0];
    }

    /**
     * 次に表示するビューを取得します。
     * @param {any} nowView ビュー
     * @returns {void} void
     */
    function getNextView(nowView) {
        var views = !getParameterByName("autoViewId")
            ? allViews
                .filter(function (av) {
                    return av.views.length > 0;
                })
                .map(function (av) {
                    return av.views;
                })
                .reduce(function (a, b) {
                    return a.concat(b);
                })
            : viewList;
        var viewListWithoutCycleDisplay = _.filter(views, function (v) {
            return v.isCycleDisplay;
        });
        if (!nowView) {
            return viewListWithoutCycleDisplay.length > 0
                ? viewListWithoutCycleDisplay[0]
                : views[0];
        }

        // ビューIDの指定が存在する場合は固定
        let viewId = getParameterByName("viewId");

        if (viewId !== null) {
            let targetViews = _.filter(viewList, function (v) {
                return v.viewId == viewId;
            }),
                queryString = parseQueryStringToDictionary(
                    location.href.split("?")[1] || ""
                ),
                params = [];

            for (var key in queryString) {
                if (key === "autoViewId" || key === "viewId") continue;

                params.push({
                    name: decodeURIComponent(key),
                    value: decodeURIComponent(queryString[key]),
                });
            }

            // パラメータが合致するものがあれば、そのビューを返す。
            for (var i = 0; i < targetViews.length; i++) {
                // パラメータの並び順がソート済み必要性有り
                if (
                    JSON.stringify(params) === JSON.stringify(targetViews[i].parameters)
                ) {
                    return targetViews[i];
                }
            }
        }

        let index = _.findIndex(viewListWithoutCycleDisplay, function (v) {
            return nowView.viewId == v.viewId &&
                JSON.stringify(nowView.parameters) == JSON.stringify(v.parameters);
        });
        for (let i = 0; i < viewListWithoutCycleDisplay.length; i++) {
            if (index < i) {
                return viewListWithoutCycleDisplay[i];
            }
        }

        return viewListWithoutCycleDisplay[0];
    }

    /**
     * レイアウトの調整を行います。
     */
    function adjustLayout() {
        var mainCanvas = document.getElementById("mainCanvas"),
            items = document.getElementsByTagName("canvas");

        for (var i = 0; i < items.length; i++) {
            if (items[i].id.startsWith("child")) {
                fitChildCanvas(items[i], mainCanvas);
                continue;
            }

            fit(items[i], items[i].parentNode);
        }
        items = document.getElementsByClassName("linkButton");
        for (var i = 0; i < items.length; i++) {
            linkButtonFit(items[i], mainCanvas);
        }
        items = document.getElementsByClassName("cameraPart");
        for (var i = 0; i < items.length; i++) {
            cameraFit(items[i], mainCanvas);
        }
        items = document.getElementById("tableDiv").children;
        for (var i = 0; i < items.length; i++) {
            tableFit(items[i], items[i].parentNode, mainCanvas);
        }
        for (var i = 0; i < tables.length; i++) {
            tables[i].fnAdjustColumnSizing();
        }
        for (var i = 0; i < tables.length; i++) {
            let table = tables[i].get(0);
            adjustWidthTableOfHeader(table);
            table.style.width = "100%";
        }
        for (var i = 0; i < items.length; i++) {
            tableFit(items[i], items[i].parentNode, mainCanvas);
        }
    }

    /**
     * 要素の大きさを調整します。
     */
    function fit(element, parent) {
        var pw = parent.clientWidth,
            ph = parent.clientHeight,
            l = 0,
            t = 0,
            w = pw,
            h = ph,
            s = element.style,
            aspectRatio = element.width / element.height;

        // 位置とサイズを計算
        if (aspectRatio < pw / ph) {
            w = (ph * aspectRatio) | 0;
        } else {
            h = (pw / aspectRatio) | 0;
        }
        l = pw > w ? (pw - w) / 2 : 0;
        t = ph > h ? (ph - h) / 2 : 0;

        // 要素のスタイルを変更
        s.left = l + "px";
        s.top = t + "px";
        s.width = w + "px";
        s.height = h + "px";
    }

    /**
     * 子キャンバスの大きさを調整します。
     * @param {any} childCanvas
     * @param {any} parentCanvas
     */
    function fitChildCanvas(childCanvas, parentCanvas) {
        var s = childCanvas.style,
            cOrgWidth = parentCanvas.width,
            cOrgHeight = parentCanvas.height,
            cWidth = parseInt(parentCanvas.clientWidth),
            cHeight = parseInt(parentCanvas.clientHeight),
            cLeft = parseInt(parentCanvas.style.left),
            cTop = parseInt(parentCanvas.style.top);

        // 比率
        var widthRatio = cWidth / cOrgWidth;
        var heightRatio = cHeight / cOrgHeight;

        // 要素のスタイルを変更
        s.left = childCanvas.dataset.left * widthRatio + cLeft + "px";
        s.top = childCanvas.dataset.top * widthRatio + cTop + "px";
        s.width = childCanvas.dataset.width * widthRatio + "px";
        s.height = childCanvas.dataset.height * heightRatio + "px";
    }

    /**
     * 遷移パーツの大きさを調整します。
     */
    function linkButtonFit(element, canvas) {
        var s = element.style,
            cOrgWidth = canvas.width,
            cOrgHeight = canvas.height,
            cWidth = parseInt(canvas.style.width),
            cHeight = parseInt(canvas.style.height),
            cLeft = parseInt(canvas.style.left),
            cTop = parseInt(canvas.style.top);

        // 比率
        var widthRatio = cWidth / cOrgWidth;
        var heightRatio = cHeight / cOrgHeight;

        // 要素のスタイルを変更
        s.left = element.dataset.left * widthRatio + cLeft + "px";
        s.top = element.dataset.top * heightRatio + cTop + "px";
        s.width = element.dataset.width * widthRatio + "px";
        s.height = element.dataset.height * heightRatio + "px";
    }

    /**
     * テーブルの大きさを調整します。
     */
    function tableFit(element, parent, canvas) {
        var s = element.style,
            cOrgWidth = canvas.width,
            cOrgHeight = canvas.height,
            cWidth = parseInt(canvas.style.width),
            cHeight = parseInt(canvas.style.height),
            cLeft = parseInt(canvas.style.left),
            cTop = parseInt(canvas.style.top);

        // 比率
        var widthRatio = cWidth / cOrgWidth;
        var heightRatio = cHeight / cOrgHeight;

        // 要素のスタイルを変更
        s.left = element.left * widthRatio + cLeft + "px";
        s.top = element.top * heightRatio + cTop + "px";
        s.width = element.width * widthRatio + "px";
        s.height = element.height * heightRatio + "px";

        // 文字サイズ
        var fSize = parseInt(s.fontSize);
        var tds = document.getElementsByTagName("td");
        for (var i = 0; i < tds.length; i++) {
            tds[i].style.fontSize = fSize * widthRatio + "px";
        }
        var ths = document.getElementsByTagName("th");
        for (var i = 0; i < ths.length; i++) {
            ths[i].style.fontSize = fSize * widthRatio + "px";
        }

        var head = $("#" + element.id + " .dataTables_scrollHead");
        var hHeight = head.height();
        var body = $("#" + element.id + " .dataTables_scrollBody");
        body.height(element.clientHeight - hHeight);
    }

    /**
     * カメラの大きさを調整します。
     */
    function cameraFit(element, canvas) {
        var s = element.style,
            cOrgWidth = canvas.width,
            cOrgHeight = canvas.height,
            cWidth = parseInt(canvas.style.width),
            cHeight = parseInt(canvas.style.height),
            cLeft = parseInt(canvas.style.left),
            cTop = parseInt(canvas.style.top);

        // 比率
        var widthRatio = cWidth / cOrgWidth;
        var heightRatio = cHeight / cOrgHeight;

        // 要素のスタイルを変更
        s.left = element.dataset.left * widthRatio + cLeft + "px";
        s.top = element.dataset.top * heightRatio + cTop + "px";
        s.width = element.dataset.width * widthRatio + "px";
        s.height = element.dataset.height * heightRatio + "px";

        var iFrame = element.children[0];
        iFrame.style.width = s.width;
        iFrame.style.height = s.height;
    }

    /**
     * 表のヘッダの幅を調整
     * @param {HTMLElement} table テーブル
     */
    function adjustWidthTableOfHeader(table) {
        var wrapper = document.querySelector("#" + table.id + "_wrapper");
        var headerInTable = wrapper.querySelector(".dataTables_scrollHeadInner");
        var tableInHeaderInTable = headerInTable.querySelector("table");
        if (wrapper.scrollWidth >= parseInt(headerInTable.style.width)) {
            headerInTable.classList.add("full-width");
            tableInHeaderInTable.classList.add("full-width");
        }
    }

    /**
     * ビューのJSファイルのURLを生成します。
     */
    function createJsUrl(view) {
        var parameterString = "";
        $.each(view.parameters, function (index, parameter) {
            parameterString += "_"; // SEPARATOR.
            parameterString += parameter.name + "=" + parameter.value;
        });
        return (
            "views/" +
            view.name +
            "_" +
            view.viewId +
            parameterString +
            ".js?" +
            new Date().getTime()
        );
    }

    /**
     * リンクを生成します。
     */
    function createLinkUrl(autoViewId, view) {
        var url = location.href.replace(location.search, "");

        url = IB_SKIN.updateQueryStringParameter(url, "autoViewId", autoViewId);
        url = IB_SKIN.updateQueryStringParameter(url, "viewId", view.viewId);
        $.each(view.parameters, function (index, parameter) {
            url = IB_SKIN.updateQueryStringParameter(
                url,
                encodeURIComponent(parameter.name),
                encodeURIComponent(parameter.value)
            );
        });
        return url;
    }

    /**
     * メニューを生成します。
     */
    function createMenu() {
        //画面切り替えメニュー
        var url = location.href.replace(location.search, ""),
            menu = document.getElementById("viewNavi"),
            autoViewId = getParameterByName("autoViewId"),
            viewId = getParameterByName("viewId");

        for (var i = 0; i < allViews.length; i++) {
            var li = document.createElement("li");
            li.className = "autoview";
            var ref = document.createElement("a");
            ref.textContent = allViews[i].name;
            ref.href = url + "?autoViewId=" + allViews[i].id;
            li.appendChild(ref);
            if (allViews[i].id == autoViewId && viewId == null) {
                li.style.background = "#2e8b57";
            }
            var ul = document.createElement("ul");
            for (var j = 0; j < allViews[i].views.length; j++) {
                var _li = document.createElement("li"),
                    link = document.createElement("a"),
                    view = allViews[i].views[j];
                _li.className = "view";

                // ビューの名に、パラメータもあればパラメータも付与する。
                if (view.parameters) {
                    var parameterText = view.parameters.map(function (p, index) {
                        var text =
                            '<small class="title--parameter">' +
                            p.name.replace("@", "") +
                            ": " +
                            p.value +
                            "</small>";
                        return "<br>" + text;
                    });
                    link.innerHTML = view.name + parameterText;
                } else {
                    link.textContent = view.name;
                }
                link.href = createLinkUrl(allViews[i].id, view);
                if (paramView && paramView.id === view.id) {
                    _li.style.background = "#2e8b57";
                }
                _li.appendChild(link);
                ul.appendChild(_li);
            }
            li.appendChild(ul);
            menu.appendChild(li);
        }
    }

    function resetCreateJSEventHandler() {
        createjs.Ticker.reset();
        createjs.Ticker._timerId = null;
        createjs.Ticker._inited = false;

        createjs.Ticker.init();
        createjs.Ticker.addEventListener("tick", createjs.Tween);
    }
})(window);

/**
 *  IB-SKIN Functions
 */
var IB_SKIN = IB_SKIN || {};

IB_SKIN.ImageContainer = (function () {
    "use strict";

    /**
     * Queue
     */
    var queueLoader = new createjs.LoadQueue(false);

    /**
     * 最大コネクション数
     */
    queueLoader.setMaxConnections(10);

    /**
     * Bitmapオブジェクトを取得します。
     * @param {any} id id
     * @param {any} src 画像のファイルパス
     * @param {any} onComplete コールバック関数
     * @returns {any} Bitmap
     */
    queueLoader.getBitmap = function (id, src, onComplete) {
        var item = queueLoader.getItem(id);
        var isContainsQueue = item != null;
        var result = queueLoader.getResult(id);
        var isLoaded = result != null;
        var image = isLoaded
            ? new createjs.Bitmap(result)
            : new createjs.Bitmap(src);
        image.visible = false;

        if (!isLoaded) {
            queueLoader.addEventListener("fileload", function (e) {
                if (e.item.id !== id) {
                    return;
                }
                image.image = queueLoader.getResult(id);
                onComplete(image);
            });
            if (!isContainsQueue) {
                queueLoader.loadFile({ id: id, src: src, type: createjs.Types.IMAGE });
            }
        } else {
            if (onComplete !== null) {
                onComplete(image);
            }
        }

        return image;
    };

    /**
     * 完了イベント
     */
    queueLoader.addEventListener("complete", function () {
        queueLoader.removeAllEventListeners("fileload");
        stage.update();
    });

    return queueLoader;
})();

/**
 * Update Querystring.
 */
IB_SKIN.updateQueryStringParameter = function (uri, key, value) {
    var re = new RegExp("([?&])" + key + "=.*?(&|$)", "i");
    var separator = uri.indexOf("?") !== -1 ? "&" : "?";
    if (uri.match(re)) {
        return uri.replace(re, "$1" + key + "=" + value + "$2");
    } else {
        return uri + separator + key + "=" + value;
    }
};

IB_SKIN.scrollTable = function (id, index) {
    setTimeout(function () {
        if (index <= 0) return false;

        var $table = $("#" + id + "_wrapper .dataTables_scrollBody"),
            $tbody = $table.find("tbody"),
            $row = $tbody.find("tr").eq(index);

        if ($row.length) $table.scrollTop($row.position().top);
    }, 100);
};

IB_SKIN.TIMERS_FOR_SCROLL = [];
IB_SKIN.clearAutoScrollTimer = function () {
    var length = IB_SKIN.TIMERS_FOR_SCROLL.length;
    for (var i = 0; i < length; i++) {
        clearInterval(IB_SKIN.TIMERS_FOR_SCROLL.pop());
    }
};

IB_SKIN.autoScrollTable = function (id, interval, rowCount) {
    $(function () {
        var timerId = setInterval(function () {
            var $table = $("#" + id + "_wrapper .dataTables_scrollBody"),
                $tbody = $table.find("tbody"),
                totalRowCount = $tbody.find("tr").length,
                nowOffset = parseInt($table.attr("data-autoscroll-now-offset")),
                nextOffset = 0,
                top = 0,
                tableHeight = $table.height(),
                rowHeight = $tbody.find("tr").first().height();

            if (isNaN(nowOffset) || !nowOffset) {
                $table.attr("data-autoscroll-now-offset", 0);
                nowOffset = 0;
            }

            let isBottom = (function () {
                var scrollTop = $table.scrollTop();
                return scrollTop + tableHeight >= $tbody.height();
            })();

            // Go to Top.
            if (isBottom) {
                $table.scrollTop(0);
                $table.attr("data-autoscroll-now-offset", 0);
                return;
            }
            nextOffset = nowOffset + rowCount;

            if (totalRowCount <= nextOffset) {
                // Go to Bottom
                $table.scrollTop($tbody.find("tr").last().position().top);
            } else {
                // Go to Next
                $table.scrollTop($tbody.find("tr").eq(nextOffset).position().top);
            }

            $table.attr("data-autoscroll-now-offset", nextOffset);
        }, interval * 1000);

        IB_SKIN.TIMERS_FOR_SCROLL.push(timerId);
    });
};
