﻿if (!String.prototype.startsWith) {
    String.prototype.startsWith = function (searchString, position) {
        position = position || 0;
        return this.substr(position, searchString.length) === searchString;
    };
}

/**
 * クエリ文字列を解析します。
 * @param {String} name
 * @param {String} url
 */
function getParameterByName(name, url) {
    if (!url) url = window.location.href;
    name = name.replace(/[\[\]]/g, "\\$&");
    var regex = new RegExp("[?&]" + name + "(=([^&#]*)|&|#|$)"),
        results = regex.exec(url);
    if (!results) return null;
    if (!results[2]) return '';
    return decodeURIComponent(results[2].replace(/\+/g, " "));
}

/**
 * クエリ文字列を辞書にマップします。
 * @param {any} queryString
 */
function parseQueryStringToDictionary(queryString) {
    var dictionary = {};

    // remove the '?' from the beginning of the
    // if it exists
    if (queryString.indexOf('?') === 0) {
        queryString = queryString.substr(1);
    }

    // Step 1: separate out each key/value pair
    var parts = queryString.split('&');

    for (var i = 0; i < parts.length; i++) {
        var p = parts[i];
        // Step 2: Split Key/Value pair
        var keyValuePair = p.split('=');

        // Step 3: Add Key/Value pair to Dictionary object
        var key = keyValuePair[0];
        var value = keyValuePair[1];

        // decode URI encoded string
        value = decodeURIComponent(value);
        value = value.replace(/\+/g, ' ');

        dictionary[key] = value;
    }

    // Step 4: Return Dictionary Object
    return dictionary;
}


/**
 * エスケープします。
 * @param {any} text エスケープ対象文字列
 * @returns {String} text エスケープされた対象文字列
 */
function escape(text) {
    return he.encode(text);
}

/**
 * アンエスケープします。
 * @param {String} text アンエスケープ対象文字列
 * @returns {String} text アンエスケープされた対象文字列
 */
function unescape(text) {
    return he.decode(text);
}

/**
 * イベントを発火します。
 * @param {any} element 要素
 * @param {any} event イベント
 * @returns {void}
 */
function triggerEvent(element, event) {
    if (document.createEvent) {
        // Not IE
        var evt = document.createEvent("MouseEvents");
        evt.initMouseEvent(event.type
            , false
            , true
            , event.view
            , event.detail
            , event.screenX
            , event.screenY
            , event.clientX
            , event.clientY
            , event.ctrKey
            , event.altKey
            , event.shiftKey
            , event.metaKey
            , event.button
            , event.relatedTarget
        );
        return element.dispatchEvent(evt);
    }

    // IE
    var evt = document.createEventObject();
    return element.fireEvent("on" + event, evt)
}

/**
 * Hex To RGB
 * @param {any} hex HEX
 * @return {any} RGB
 */
function hexToRgb(hex) {
    var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return result ? {
        r: parseInt(result[1], 16),
        g: parseInt(result[2], 16),
        b: parseInt(result[3], 16)
    } : null;
}



/**
 * CreateJS Helper
 * */
this.CreateJSHelper = this.CreateJSHelper || {};

/**
* @description
* `Text`クラスのインスタンス`textInstance`の`text`プロパティに、
* `textInstance`の幅をはみ出さないように、自動で折り返した`text`を格納する。
* @param {Text} textInstance 自動で折り返した`text`を格納する`Text`クラスのインスタンス
*/
this.CreateJSHelper.setWrapText = function (textInstance) {
    if (textInstance.lineWidth === null) {
        return;
    }

    var initWidth = textInstance.lineWidth;
    var text = textInstance.text;
    var textArray = text.split('');
    var i = -1;
    var prevText = '';
    var lines = [];

    textInstance.text = '';

    while (textArray[++i]) {
        textInstance.text += textArray[i];

        if (textInstance.getMeasuredWidth() > initWidth) {
            lines.push(prevText);
            textInstance.text = textArray[i];
        }
        prevText = textInstance.text;
    }

    lines.push(prevText);
    textInstance.text = lines.join('\n');

    if (lines.length === 0) {
        textInstance.text = text;
    }
};
 